/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg.queryhintsexample;

import jakarta.persistence.QueryHint;
import java.util.List;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.jpa.repository.QueryHints;
import org.springframework.stereotype.Repository;

/**
 *
 * @author omozegieaziegbe
 */
@Repository
public interface BookRepository extends JpaRepository<Book, Long> {

    @QueryHints(value = {
        @QueryHint(name = "org.hibernate.fetchSize", value = "20")})
    List<Book> findBookByTitle(String title);

    @QueryHints({
        @QueryHint(name = "javax.persistence.lock.timeout", value = "5000") // 5 seconds
    })
    List<Book> findAllBooks();

    @QueryHints({
        @QueryHint(name = "org.hibernate.cacheable", value = "true")
    })
    List<Book> findAllCachedBooks();

    @QueryHints({
        @QueryHint(name = "org.hibernate.readOnly", value = "true")
    })
    List<Book> findReadOnlyBooks(double amountThreshold);

    @Query(value = "SELECT b FROM Book b", nativeQuery = true)
    @QueryHints({
        @QueryHint(name = "org.hibernate.comment", value = "My Custom Comment")
    })
    List<Book> findAllWithCustomBooks();

    @QueryHints({
        @QueryHint(name = "javax.persistence.query.timeout", value = "5000") // 5 seconds
    })
    List<Book> findAllBooksWithQueryTimeout();
    
    @QueryHints({
        @QueryHint(name = "jakarta.persistence.cache.storeMode", value = "REFRESH")
    })
    List<Book> findAllBooksWithCacheStoreMode();
    
    @QueryHints({
        @QueryHint(name = "jakarta.persistence.cache.retrieveMode", value = "USE")
    })
    List<Book> findAllBooksWithCacheRetrieveMode();
}
